from girder import events
from girder.constants import SortDir
from girder.exceptions import ValidationException
from girder.models.user import User
from girder.plugin import GirderPlugin

from . import rest, providers


def checkOidcUser(event):
    """
    If an OIDC user without a password tries to log in with a password, we
    want to give them a useful error message.
    """
    user = event.info['user']
    if user.get('oidc'):
        raise ValidationException(
            "You don't have a password. Please log in with OIDC, or use the password reset link.")


class OidcPlugin(GirderPlugin):
    DISPLAY_NAME = 'OIDC/Keycloak Login'
    CLIENT_SOURCE_PATH = 'web_client'
    description = 'Authenticate users via OIDC (Keycloak)'

    def load(self, info):
        User().ensureIndex((
            (('oidc.provider', SortDir.ASCENDING),
             ('oidc.id', SortDir.ASCENDING)), {}))

        events.bind('no_password_login_attempt', 'oidc', checkOidcUser)

        info['apiRoot'].oidc = rest.Oidc()



"""Tests for Keycloak provider."""

import os
import pytest
from unittest.mock import Mock, patch, MagicMock
from girder.models.setting import Setting
from girder.exceptions import RestException
from girder.models.user import User

from girder_oidc.providers import KeycloakProvider
from girder_oidc.settings import PluginSettings


class TestKeycloakProvider:
    """Test Keycloak OIDC provider."""

    def setup_method(self):
        """Set up test fixtures."""
        # Clear environment variables
        for key in ['KEYCLOAK_URL', 'KEYCLOAK_PUBLIC_URL']:
            if key in os.environ:
                del os.environ[key]

    def test_init_from_parameters(self):
        """Test provider initialization with explicit parameters."""
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        assert provider.keycloakUrl == 'https://keycloak:8443'
        assert provider.keycloakPublicUrl == 'https://localhost:8443'
        assert provider.realm == 'test'
        assert provider.clientId == 'client'

    def test_init_from_env_variables(self):
        """Test provider initialization from environment variables."""
        os.environ['KEYCLOAK_URL'] = 'https://env-keycloak:8443'
        os.environ['KEYCLOAK_PUBLIC_URL'] = 'https://env-public:8443'
        
        provider = KeycloakProvider()
        
        assert provider.keycloakUrl == 'https://env-keycloak:8443'
        assert provider.keycloakPublicUrl == 'https://env-public:8443'

    def test_url_normalization(self):
        """Test that URLs are normalized (trailing slashes removed)."""
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443/',
            keycloakPublicUrl='https://localhost:8443/',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        assert provider.keycloakUrl == 'https://keycloak:8443'
        assert provider.keycloakPublicUrl == 'https://localhost:8443'

    def test_realm_url_construction(self):
        """Test realm URL is constructed correctly."""
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='my-realm',
            clientId='client',
            clientSecret='secret'
        )
        
        assert provider.realmUrl == 'https://keycloak:8443/realms/my-realm'
        assert provider.realmPublicUrl == 'https://localhost:8443/realms/my-realm'

    @patch('girder_oidc.providers.requests.get')
    def test_get_well_known_config(self, mock_get):
        """Test fetching well-known OIDC configuration."""
        mock_response = Mock()
        mock_response.json.return_value = {
            'authorization_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/auth',
            'token_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/token',
            'userinfo_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/userinfo',
        }
        mock_get.return_value = mock_response
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        config = provider.getWellKnownConfig()
        
        assert 'authorization_endpoint' in config
        assert 'token_endpoint' in config
        mock_get.assert_called_once()

    @patch('girder_oidc.providers.requests.get')
    def test_get_authorization_url(self, mock_get):
        """Test authorization URL generation."""
        mock_response = Mock()
        mock_response.json.return_value = {
            'authorization_endpoint': 'https://localhost:8443/realms/test/protocol/openid-connect/auth',
            'token_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/token',
            'userinfo_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/userinfo',
        }
        mock_get.return_value = mock_response
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='my-client',
            clientSecret='secret'
        )
        
        auth_url = provider.getAuthorizationUrl('state123', 'https://girder/callback')
        
        assert 'client_id=my-client' in auth_url
        assert 'state=state123' in auth_url
        assert 'redirect_uri=' in auth_url
        assert 'https://localhost:8443' in auth_url  # Should use public URL

    @patch('girder_oidc.providers.requests.post')
    @patch('girder_oidc.providers.requests.get')
    def test_get_token(self, mock_get, mock_post):
        """Test token exchange."""
        mock_get.return_value.json.return_value = {
            'token_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/token',
            'authorization_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/auth',
            'userinfo_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/userinfo',
        }
        
        mock_post.return_value.json.return_value = {
            'access_token': 'token123',
            'token_type': 'Bearer',
            'expires_in': 3600,
        }
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        token_data = provider.getToken('code123', 'https://girder/callback')
        
        assert token_data['access_token'] == 'token123'
        mock_post.assert_called_once()

    @patch('girder_oidc.providers.requests.get')
    def test_get_user_info(self, mock_get):
        """Test fetching user information."""
        mock_get.side_effect = [
            Mock(json=lambda: {
                'token_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/token',
                'authorization_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/auth',
                'userinfo_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/userinfo',
            }),
            Mock(json=lambda: {
                'sub': 'user123',
                'email': 'user@example.com',
                'given_name': 'John',
                'family_name': 'Doe',
            })
        ]
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        user_info = provider.getUserInfo('token123')
        
        assert user_info['sub'] == 'user123'
        assert user_info['email'] == 'user@example.com'
        assert user_info['given_name'] == 'John'

    @patch('girder_oidc.providers.requests.get')
    def test_fix_url_for_public(self, mock_get):
        """Test URL replacement for public URLs."""
        mock_get.return_value.json.return_value = {
            'authorization_endpoint': 'https://keycloak:8443/realms/test/auth',
            'token_endpoint': 'https://keycloak:8443/realms/test/token',
            'userinfo_endpoint': 'https://keycloak:8443/realms/test/userinfo',
        }
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        fixed_url = provider._fixUrlForPublic('https://keycloak:8443/realms/test/auth')
        assert fixed_url == 'https://localhost:8443/realms/test/auth'

    def test_fix_url_for_public_no_change(self):
        """Test URL replacement when URLs are the same."""
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://keycloak:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        original_url = 'https://keycloak:8443/realms/test/auth'
        fixed_url = provider._fixUrlForPublic(original_url)
        assert fixed_url == original_url



import json
import os
import requests
from urllib.parse import urlencode

from girder.api.rest import getApiUrl
from girder.exceptions import RestException, ValidationException
from girder.models.setting import Setting
from girder.models.user import User
from girder.settings import SettingKey

from ..settings import PluginSettings


def _getVerifySetting():
    """
    Get the SSL verification setting from environment.
    
    Returns True (use system certs) if REQUESTS_CA_BUNDLE is not set or points to a non-existent file.
    Returns the path if it exists and is readable.
    """
    ca_bundle = os.environ.get('REQUESTS_CA_BUNDLE')
    if ca_bundle and os.path.isfile(ca_bundle):
        return ca_bundle
    return True


class KeycloakProvider:
    """
    OIDC provider for Keycloak.
    
    This provider implements the OpenID Connect protocol for authentication
    with a Keycloak instance.
    """
    
    def __init__(self, keycloakUrl=None, keycloakPublicUrl=None, realm=None, clientId=None, clientSecret=None):
        """
        Initialize the Keycloak OIDC provider.
        
        :param keycloakUrl: Internal Keycloak URL (for server-to-server communication)
        :param keycloakPublicUrl: Public Keycloak URL (for browser redirects)
        :param realm: Keycloak realm name
        :param clientId: OIDC client ID
        :param clientSecret: OIDC client secret
        """
        # Check parameters first, then environment variables, then database settings
        self.keycloakUrl = keycloakUrl or os.environ.get('KEYCLOAK_URL') or Setting().get(PluginSettings.KEYCLOAK_URL)
        # Public URL: prefer parameter, then env var, then setting, then fall back to internal URL
        self.keycloakPublicUrl = keycloakPublicUrl or os.environ.get('KEYCLOAK_PUBLIC_URL') or Setting().get(PluginSettings.KEYCLOAK_PUBLIC_URL) or self.keycloakUrl
        
        self.realm = realm or Setting().get(PluginSettings.KEYCLOAK_REALM)
        self.clientId = clientId or Setting().get(PluginSettings.CLIENT_ID)
        self.clientSecret = clientSecret or Setting().get(PluginSettings.CLIENT_SECRET)
        
        # Normalize URLs
        self.keycloakUrl = self.keycloakUrl.rstrip('/')
        self.keycloakPublicUrl = self.keycloakPublicUrl.rstrip('/')
        self.realmUrl = f'{self.keycloakUrl}/realms/{self.realm}'
        self.realmPublicUrl = f'{self.keycloakPublicUrl}/realms/{self.realm}'
    
    def _fixUrlForPublic(self, url):
        """
        Replace internal Keycloak URL with public URL in a given URL string.
        
        :param url: URL that may contain the internal Keycloak URL
        :returns: URL with public URL substituted if needed
        """
        if self.keycloakUrl != self.keycloakPublicUrl and url:
            return url.replace(self.keycloakUrl, self.keycloakPublicUrl)
        return url
        
    def getWellKnownConfig(self):
        """
        Fetch the OpenID Connect well-known configuration from Keycloak.
        
        :returns: Configuration dictionary with endpoints and capabilities
        """
        url = f'{self.realmUrl}/.well-known/openid-configuration'
        try:
            resp = requests.get(url, timeout=10, verify=_getVerifySetting())
            resp.raise_for_status()
            return resp.json()
        except requests.RequestException as e:
            raise RestException(f'Failed to fetch Keycloak config: {str(e)}', code=502)
    
    def getAuthorizationUrl(self, state, redirectUri):
        """
        Get the Keycloak authorization URL for initiating the OIDC flow.
        
        :param state: CSRF state token
        :param redirectUri: Callback URI
        :returns: Authorization URL
        """
        config = self.getWellKnownConfig()
        authUrl = self._fixUrlForPublic(config['authorization_endpoint'])
        
        params = {
            'client_id': self.clientId,
            'response_type': 'code',
            'scope': 'openid profile email',
            'redirect_uri': redirectUri,
            'state': state,
        }
        
        return f'{authUrl}?{urlencode(params)}'
    
    def getToken(self, code, redirectUri):
        """
        Exchange authorization code for access token.
        
        :param code: Authorization code from Keycloak
        :param redirectUri: Callback URI
        :returns: Token response dictionary
        """
        config = self.getWellKnownConfig()
        tokenUrl = config['token_endpoint']
        
        payload = {
            'grant_type': 'authorization_code',
            'code': code,
            'client_id': self.clientId,
            'client_secret': self.clientSecret,
            'redirect_uri': redirectUri,
        }
        
        try:
            resp = requests.post(tokenUrl, data=payload, timeout=10, verify=_getVerifySetting())
            resp.raise_for_status()
            token_data = resp.json()
            return token_data
        
        except requests.RequestException as e:
            raise RestException(f'Failed to get token: {str(e)}', code=502)
    
    def getUserInfo(self, accessToken):
        """
        Fetch user information from Keycloak using the access token.
        
        :param accessToken: OIDC access token
        :returns: User information dictionary
        """
        config = self.getWellKnownConfig()
        userinfoUrl = config['userinfo_endpoint']
        
        headers = {'Authorization': f'Bearer {accessToken}'}
        verify_setting = _getVerifySetting()
        
        try:
            resp = requests.get(userinfoUrl, headers=headers, timeout=10, verify=verify_setting)
            resp.raise_for_status()
            user_data = resp.json()
            return user_data
        except requests.RequestException as e:
            raise RestException(f'Failed to get user info: {str(e)}', code=502)
    
    def createOrUpdateUser(self, userInfo):
        """
        Create or update a Girder user based on OIDC user information.
        
        :param userInfo: User information from Keycloak
        :returns: Girder user document
        """
        oauthId = userInfo.get('sub')
        email = userInfo.get('email')
        firstName = userInfo.get('given_name', '')
        lastName = userInfo.get('family_name', '')
        
        if not oauthId:
            raise RestException('No subject claim in token', code=502)
        
        if not email:
            raise RestException('No email in user info', code=502)
        
        # Try finding by OIDC ID first
        query = {
            'oidc.provider': 'keycloak',
            'oidc.id': oauthId
        }
        user = User().findOne(query)
        
        if user:
            # Update existing user
            user['email'] = email
            user['firstName'] = firstName
            user['lastName'] = lastName
            return User().save(user)
        
        # Try finding by email
        user = User().findOne({'email': email})
        
        if not user:
            # Check if registration is allowed
            policy = Setting().get(SettingKey.REGISTRATION_POLICY)
            allowRegistration = Setting().get(PluginSettings.ALLOW_REGISTRATION)
            
            if policy == 'closed' and not allowRegistration:
                raise RestException(
                    'User registration is disabled',
                    code=403
                )
            
            # Check auto-create setting
            if not Setting().get(PluginSettings.AUTO_CREATE_USERS):
                raise RestException(
                    'User auto-creation is disabled. Please contact an administrator.',
                    code=403
                )
            
            # Create login from email prefix
            login = email.split('@')[0]
            
            # Ensure unique login
            counter = 1
            originalLogin = login
            while User().findOne({'login': login}):
                login = f'{originalLogin}{counter}'
                counter += 1
            
            # Create user
            user = User().createUser(
                login=login,
                email=email,
                firstName=firstName,
                lastName=lastName,
                password=None,  # OIDC users don't have passwords
            )
        
        # Store OIDC information
        if 'oidc' not in user:
            user['oidc'] = []
        
        # Add or update OIDC provider info
        oidcInfo = {
            'provider': 'keycloak',
            'id': oauthId,
        }
        
        # Check if this provider already exists
        user['oidc'] = [o for o in user['oidc'] if o.get('provider') != 'keycloak']
        user['oidc'].append(oidcInfo)
        
        return User().save(user)



import requests

USERNAME = 'testuser'
PASSWORD = 'testpass123'
REALM = 'girder'
CLIENT_ID = 'girder'
CLIENT_SECRET = 'qq819I9oFVYY05D4T8KOUQxis60nLBKr'
KEYCLOAK_URL = 'https://localhost:8443'
GIRDER_API_URL = 'http://localhost:8080/api/v1'

def main():
    """ We test the API endpoint to exchange an OICD token for a Girder token. """
    # First, get an OIDC token from Keycloak using direct access grant
    token_resp = requests.post(
        f'{KEYCLOAK_URL}/realms/{REALM}/protocol/openid-connect/token',
        data={
            'grant_type': 'password',
            'client_id': CLIENT_ID,
            'client_secret': CLIENT_SECRET,
            'username': USERNAME,
            'password': PASSWORD,
            'scope': 'openid profile email',  # Explicitly request scopes
        },
        verify=False  # Insecure, for testing only
    )
    token_resp.raise_for_status()
    oidc_token = token_resp.json()['access_token']
    
    # Now exchange the OIDC token for a Girder auth token
    girder_resp = requests.post(
        f'{GIRDER_API_URL}/oidc/token',
        data={'access_token': oidc_token}
    )
    girder_resp.raise_for_status()
    girder_token = girder_resp.json()['authToken']
    
    print(f'Obtained Girder auth token: {girder_token}')

if __name__ == '__main__':
    main()


"""Pytest configuration for OIDC plugin tests."""

import pytest
from girder.test import base


@pytest.fixture(scope='session')
def db(tmpdir_factory):
    """Set up test database."""
    return base.startServer(
        plugins=['oidc']
    )



import os

from girder.exceptions import ValidationException
from girder.utility import setting_utilities


class PluginSettings:
    """Settings keys for OIDC plugin."""
    
    KEYCLOAK_URL = 'oidc.keycloak_url'
    KEYCLOAK_PUBLIC_URL = 'oidc.keycloak_public_url'
    KEYCLOAK_REALM = 'oidc.keycloak_realm'
    CLIENT_ID = 'oidc.client_id'
    CLIENT_SECRET = 'oidc.client_secret'
    ENABLE = 'oidc.enable'
    AUTO_CREATE_USERS = 'oidc.auto_create_users'
    ALLOW_REGISTRATION = 'oidc.allow_registration'
    IGNORE_REGISTRATION_POLICY = 'oidc.ignore_registration_policy'
    GROUP_MAPPING = 'oidc.group_mapping'


@setting_utilities.default(PluginSettings.KEYCLOAK_URL)
def _defaultKeycloakUrl():
    return os.environ.get('KEYCLOAK_URL', 'https://keycloak:8443')


@setting_utilities.default(PluginSettings.KEYCLOAK_PUBLIC_URL)
def _defaultKeycloakPublicUrl():
    return os.environ.get('KEYCLOAK_PUBLIC_URL', os.environ.get('KEYCLOAK_URL', 'https://localhost:8443'))


@setting_utilities.default(PluginSettings.KEYCLOAK_REALM)
def _defaultKeycloakRealm():
    return 'girder'


@setting_utilities.default(PluginSettings.CLIENT_ID)
def _defaultClientId():
    return ''


@setting_utilities.default(PluginSettings.CLIENT_SECRET)
def _defaultClientSecret():
    return ''


@setting_utilities.default(PluginSettings.ENABLE)
def _defaultEnable():
    return False


@setting_utilities.default(PluginSettings.AUTO_CREATE_USERS)
def _defaultAutoCreateUsers():
    return True


@setting_utilities.default(PluginSettings.ALLOW_REGISTRATION)
def _defaultAllowRegistration():
    return False


@setting_utilities.default(PluginSettings.IGNORE_REGISTRATION_POLICY)
def _defaultIgnoreRegistrationPolicy():
    return False


@setting_utilities.default(PluginSettings.GROUP_MAPPING)
def _defaultGroupMapping():
    return {}


@setting_utilities.validator({
    PluginSettings.KEYCLOAK_URL,
    PluginSettings.KEYCLOAK_PUBLIC_URL,
    PluginSettings.KEYCLOAK_REALM,
    PluginSettings.CLIENT_ID,
    PluginSettings.CLIENT_SECRET,
})
def _validateStringSettings(doc):
    if not isinstance(doc['value'], str):
        raise ValidationException('Value must be a string.')


@setting_utilities.validator({
    PluginSettings.ENABLE,
    PluginSettings.AUTO_CREATE_USERS,
    PluginSettings.ALLOW_REGISTRATION,
    PluginSettings.IGNORE_REGISTRATION_POLICY,
})
def _validateBooleanSettings(doc):
    if not isinstance(doc['value'], bool):
        raise ValidationException('Value must be a boolean.')


@setting_utilities.validator(PluginSettings.GROUP_MAPPING)
def _validateGroupMapping(doc):
    if not isinstance(doc['value'], dict):
        raise ValidationException('Value must be a dictionary.')



"""Tests for OIDC REST endpoints."""

import json
from unittest.mock import Mock, patch
from girder.test import base
from girder.models.setting import Setting
from girder.models.token import Token

from girder_oidc.settings import PluginSettings


class TestOidcRestEndpoints(base.TestCase):
    """Test OIDC REST API endpoints."""

    def setUp(self):
        """Set up test fixtures."""
        super().setUp()
        self.admin_user = self.model('user').createUser(
            login='admin', email='admin@example.com', firstName='Admin',
            lastName='User', password='password')
        self.user = self.model('user').createUser(
            login='user', email='user@example.com', firstName='Regular',
            lastName='User', password='password')
        self.ensureUserIsAdmin(self.admin_user)
        
        # Configure OIDC
        Setting().set(PluginSettings.KEYCLOAK_URL, 'https://keycloak:8443')
        Setting().set(PluginSettings.KEYCLOAK_PUBLIC_URL, 'https://localhost:8443')
        Setting().set(PluginSettings.KEYCLOAK_REALM, 'test')
        Setting().set(PluginSettings.CLIENT_ID, 'test-client')
        Setting().set(PluginSettings.ENABLE, True)

    @patch('girder_oidc.providers.requests.get')
    def testGetLoginUrl(self, mock_get):
        """Test GET /oidc/login returns authorization URL."""
        mock_get.return_value.json.return_value = {
            'authorization_endpoint': 'https://localhost:8443/realms/test/protocol/openid-connect/auth',
            'token_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/token',
            'userinfo_endpoint': 'https://keycloak:8443/realms/test/protocol/openid-connect/userinfo',
        }
        
        resp = self.request('/oidc/login', query={'redirect': 'http://localhost:8080/#!/login'})
        self.assertStatusOk(resp)
        
        data = resp.json
        self.assertIn('url', data)
        self.assertIn('authorization_endpoint', data['url'])
        self.assertIn('client_id=test-client', data['url'])
        self.assertIn('response_type=code', data['url'])

    def testGetLoginUrlNotEnabled(self):
        """Test login fails when OIDC is not enabled."""
        Setting().set(PluginSettings.ENABLE, False)
        
        resp = self.request('/oidc/login', query={'redirect': 'http://localhost:8080'})
        self.assertStatus(resp, 403)

    def testGetLoginUrlNoClientId(self):
        """Test login fails without client ID configured."""
        Setting().set(PluginSettings.CLIENT_ID, '')
        
        resp = self.request('/oidc/login', query={'redirect': 'http://localhost:8080'})
        self.assertStatus(resp, 500)

    @patch('girder_oidc.providers.requests.post')
    @patch('girder_oidc.providers.requests.get')
    def testCallbackMissingCode(self, mock_get, mock_post):
        """Test callback fails without authorization code."""
        resp = self.request('/oidc/callback', query={'state': 'state123'})
        self.assertStatus(resp, 400)

    @patch('girder_oidc.providers.requests.post')
    @patch('girder_oidc.providers.requests.get')
    def testCallbackInvalidState(self, mock_get, mock_post):
        """Test callback fails with invalid state token."""
        resp = self.request('/oidc/callback', 
                           query={'state': 'invalid.token', 'code': 'code123'})
        self.assertStatus(resp, 403)

    def testIsOidcUserNotLoggedIn(self):
        """Test isOidcUser returns false for anonymous users."""
        resp = self.request('/oidc/is-oidc-user')
        self.assertStatusOk(resp)
        
        data = resp.json
        self.assertFalse(data['isOidcUser'])

    def testIsOidcUserNotOidcUser(self):
        """Test isOidcUser returns false for non-OIDC users."""
        resp = self.request('/oidc/is-oidc-user', user=self.user)
        self.assertStatusOk(resp)
        
        data = resp.json
        self.assertFalse(data['isOidcUser'])

    def testIsOidcUserOidcUser(self):
        """Test isOidcUser returns true for OIDC authenticated users."""
        # Add OIDC info to user
        self.user['oidc'] = [{'provider': 'keycloak', 'id': 'user123'}]
        self.model('user').save(self.user)
        
        resp = self.request('/oidc/is-oidc-user', user=self.user)
        self.assertStatusOk(resp)
        
        data = resp.json
        self.assertTrue(data['isOidcUser'])

    def testLoginAttemptWithoutPassword(self):
        """Test that OIDC users get helpful error when trying password login."""
        # Add OIDC info to user but no password
        self.user['oidc'] = [{'provider': 'keycloak', 'id': 'user123'}]
        self.user['password'] = ''
        self.model('user').save(self.user)
        
        # Try to login with password
        resp = self.request('/user/me', user=self.user)
        self.assertStatusOk(resp)
        # User should be logged in via token


class TestOidcSettingsValidation(base.TestCase):
    """Test OIDC settings validation."""

    def setUp(self):
        """Set up test fixtures."""
        super().setUp()
        self.admin_user = self.model('user').createUser(
            login='admin', email='admin@example.com', firstName='Admin',
            lastName='User', password='password')
        self.ensureUserIsAdmin(self.admin_user)

    def testSetConfigurationWithValidStrings(self):
        """Test setting configuration with valid string values."""
        config_data = {
            'keycloakUrl': 'https://valid.example.com',
            'keycloakPublicUrl': 'https://public.example.com',
            'keycloakRealm': 'valid-realm',
            'clientId': 'valid-client-id',
            'clientSecret': 'valid-secret',
            'enable': True,
            'autoCreateUsers': True,
            'allowRegistration': False,
            'ignoreRegistrationPolicy': False
        }
        
        resp = self.request('/oidc/configuration', method='PUT',
                           user=self.admin_user, data=config_data)
        self.assertStatusOk(resp)

    def testSetConfigurationWithValidBooleans(self):
        """Test setting configuration with boolean values."""
        config_data = {
            'keycloakUrl': 'https://keycloak:8443',
            'keycloakPublicUrl': 'https://localhost:8443',
            'keycloakRealm': 'test',
            'clientId': 'client',
            'clientSecret': 'secret',
            'enable': False,
            'autoCreateUsers': False,
            'allowRegistration': True,
            'ignoreRegistrationPolicy': True
        }
        
        resp = self.request('/oidc/configuration', method='PUT',
                           user=self.admin_user, data=config_data)
        self.assertStatusOk(resp)
        
        # Verify values were set
        result = resp.json
        self.assertFalse(result['enable'])
        self.assertFalse(result['autoCreateUsers'])
        self.assertTrue(result['allowRegistration'])
        self.assertTrue(result['ignoreRegistrationPolicy'])



"""Tests for OIDC user management."""

from unittest.mock import Mock, patch
from girder.test import base
from girder.exceptions import RestException
from girder.models.setting import Setting
from girder.models.user import User
from girder.settings import SettingKey

from girder_oidc.providers import KeycloakProvider
from girder_oidc.settings import PluginSettings


class TestOidcUserManagement(base.TestCase):
    """Test OIDC user creation and management."""

    def setUp(self):
        """Set up test fixtures."""
        super().setUp()
        # Enable auto-create by default
        Setting().set(PluginSettings.AUTO_CREATE_USERS, True)
        Setting().set(PluginSettings.ALLOW_REGISTRATION, False)

    def testCreateUserFromOidc(self):
        """Test creating a new Girder user from OIDC credentials."""
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        user_info = {
            'sub': 'oidc-user-123',
            'email': 'newuser@example.com',
            'given_name': 'New',
            'family_name': 'User'
        }
        
        user = provider.createOrUpdateUser(user_info)
        
        self.assertIsNotNone(user)
        self.assertEqual(user['email'], 'newuser@example.com')
        self.assertEqual(user['firstName'], 'New')
        self.assertEqual(user['lastName'], 'User')
        self.assertTrue(user.get('oidc'))
        self.assertEqual(user['oidc'][0]['provider'], 'keycloak')
        self.assertEqual(user['oidc'][0]['id'], 'oidc-user-123')

    def testUpdateExistingOidcUser(self):
        """Test updating an existing OIDC user."""
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        # Create initial user
        initial_info = {
            'sub': 'oidc-user-456',
            'email': 'user@example.com',
            'given_name': 'John',
            'family_name': 'Doe'
        }
        user1 = provider.createOrUpdateUser(initial_info)
        user_id = user1['_id']
        
        # Update same user with new info
        updated_info = {
            'sub': 'oidc-user-456',
            'email': 'user@example.com',
            'given_name': 'Jonathan',
            'family_name': 'Doe'
        }
        user2 = provider.createOrUpdateUser(updated_info)
        
        self.assertEqual(user2['_id'], user_id)
        self.assertEqual(user2['firstName'], 'Jonathan')
        # Should only have one OIDC entry
        self.assertEqual(len(user2['oidc']), 1)

    def testCreateUserByEmailLookup(self):
        """Test updating user found by email."""
        # Create a user without OIDC
        existing_user = self.model('user').createUser(
            login='existing', email='existing@example.com',
            firstName='Existing', lastName='User', password='password'
        )
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        # Create OIDC entry for same email
        user_info = {
            'sub': 'oidc-user-789',
            'email': 'existing@example.com',
            'given_name': 'Existing',
            'family_name': 'User'
        }
        
        user = provider.createOrUpdateUser(user_info)
        
        self.assertEqual(user['_id'], existing_user['_id'])
        self.assertTrue(user.get('oidc'))
        self.assertEqual(user['oidc'][0]['provider'], 'keycloak')

    def testCreateUserAutoCreateDisabled(self):
        """Test that creation fails when auto-create is disabled."""
        Setting().set(PluginSettings.AUTO_CREATE_USERS, False)
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        user_info = {
            'sub': 'oidc-user-new',
            'email': 'newuser@example.com',
            'given_name': 'New',
            'family_name': 'User'
        }
        
        with self.assertRaises(RestException) as cm:
            provider.createOrUpdateUser(user_info)
        
        self.assertEqual(cm.exception.code, 403)

    def testCreateUserClosedRegistration(self):
        """Test user creation respects closed registration policy."""
        Setting().set(PluginSettings.AUTO_CREATE_USERS, True)
        Setting().set(PluginSettings.ALLOW_REGISTRATION, False)
        Setting().set(SettingKey.REGISTRATION_POLICY, 'closed')
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        user_info = {
            'sub': 'oidc-user-closed',
            'email': 'newuser@example.com',
            'given_name': 'New',
            'family_name': 'User'
        }
        
        with self.assertRaises(RestException) as cm:
            provider.createOrUpdateUser(user_info)
        
        self.assertEqual(cm.exception.code, 403)

    def testCreateUserIgnoreRegistrationPolicy(self):
        """Test ignoring closed registration policy."""
        Setting().set(PluginSettings.AUTO_CREATE_USERS, True)
        Setting().set(PluginSettings.ALLOW_REGISTRATION, False)
        Setting().set(PluginSettings.IGNORE_REGISTRATION_POLICY, True)
        Setting().set(SettingKey.REGISTRATION_POLICY, 'closed')
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        user_info = {
            'sub': 'oidc-user-ignore',
            'email': 'newuser@example.com',
            'given_name': 'New',
            'family_name': 'User'
        }
        
        # Should succeed even though registration is closed
        user = provider.createOrUpdateUser(user_info)
        self.assertIsNotNone(user)

    def testCreateUserMissingEmail(self):
        """Test that user creation fails without email."""
        Setting().set(PluginSettings.AUTO_CREATE_USERS, True)
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        user_info = {
            'sub': 'oidc-user-noemail',
            'given_name': 'No',
            'family_name': 'Email'
        }
        
        with self.assertRaises(RestException) as cm:
            provider.createOrUpdateUser(user_info)
        
        self.assertEqual(cm.exception.code, 502)

    def testCreateUserMissingSubject(self):
        """Test that user creation fails without subject claim."""
        Setting().set(PluginSettings.AUTO_CREATE_USERS, True)
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        user_info = {
            'email': 'user@example.com',
            'given_name': 'Test',
            'family_name': 'User'
        }
        
        with self.assertRaises(RestException) as cm:
            provider.createOrUpdateUser(user_info)
        
        self.assertEqual(cm.exception.code, 502)

    def testUniqueLoginGeneration(self):
        """Test that login names are made unique."""
        Setting().set(PluginSettings.AUTO_CREATE_USERS, True)
        
        provider = KeycloakProvider(
            keycloakUrl='https://keycloak:8443',
            keycloakPublicUrl='https://localhost:8443',
            realm='test',
            clientId='client',
            clientSecret='secret'
        )
        
        # Create first user with email prefix
        user_info1 = {
            'sub': 'oidc-user-1',
            'email': 'john@example.com',
            'given_name': 'John',
            'family_name': 'Doe'
        }
        user1 = provider.createOrUpdateUser(user_info1)
        self.assertEqual(user1['login'], 'john')
        
        # Create another user with same email prefix
        user_info2 = {
            'sub': 'oidc-user-2',
            'email': 'john.smith@example.com',
            'given_name': 'John',
            'family_name': 'Smith'
        }
        user2 = provider.createOrUpdateUser(user_info2)
        self.assertEqual(user2['login'], 'john1')



import cherrypy
import datetime
from urllib.parse import urlparse, parse_qs, urlencode, urlunparse

from girder import events
from girder.constants import AccessType, SortDir
from girder.exceptions import RestException, ValidationException
from girder.api.describe import Description, autoDescribeRoute
from girder.api.rest import Resource, getApiUrl
from girder.api import access
from girder.models.setting import Setting
from girder.models.user import User
from girder.models.token import Token

from .providers import KeycloakProvider
from .settings import PluginSettings


class Oidc(Resource):
    """REST API endpoints for OIDC authentication."""
    
    def __init__(self):
        super().__init__()
        self.resourceName = 'oidc'
        
        self.route('GET', ('configuration',), self.getConfiguration)
        self.route('PUT', ('configuration',), self.setConfiguration)
        self.route('GET', ('login',), self.getLoginUrl)
        self.route('GET', ('callback',), self.callback)
        self.route('GET', ('is-oidc-user',), self.isOidcUser)
        self.route('POST', ('token',), self.exchangeToken)
    
    def _createStateToken(self, redirect):
        """
        Create a CSRF token for the OIDC flow.
        
        :param redirect: URL to redirect to after authentication
        :returns: State token
        """
        csrfToken = Token().createToken(days=0.25)
        
        # The delimiter is arbitrary, but a dot doesn't need to be URL-encoded
        state = f'{csrfToken["_id"]}.{redirect}'
        return state
    
    def _validateStateToken(self, state):
        """
        Validate and consume a CSRF token.
        
        :param state: State token from OIDC callback
        :returns: Redirect URL
        :raises RestException: If token is invalid or expired
        """
        csrfTokenId, _, redirect = state.partition('.')
        
        token = Token().load(csrfTokenId, objectId=False, level=AccessType.READ)
        if token is None:
            raise RestException('Invalid CSRF token', code=403)
        
        Token().remove(token)
        
        if token['expires'] < datetime.datetime.utcnow():
            raise RestException('Expired CSRF token', code=403)
        
        if not redirect:
            raise RestException('No redirect location in state', code=400)
        
        return redirect
    
    @access.admin
    @autoDescribeRoute(
        Description('Get OIDC configuration (without secrets).')
    )
    def getConfiguration(self):
        """Get OIDC configuration for the admin panel."""
        return {
            'keycloakUrl': Setting().get(PluginSettings.KEYCLOAK_URL) or '',
            'keycloakPublicUrl': Setting().get(PluginSettings.KEYCLOAK_PUBLIC_URL) or '',
            'keycloakRealm': Setting().get(PluginSettings.KEYCLOAK_REALM) or '',
            'clientId': Setting().get(PluginSettings.CLIENT_ID) or '',
            'enable': Setting().get(PluginSettings.ENABLE) or False,
            'autoCreateUsers': Setting().get(PluginSettings.AUTO_CREATE_USERS) or False,
            'allowRegistration': Setting().get(PluginSettings.ALLOW_REGISTRATION) or False,
            'ignoreRegistrationPolicy': Setting().get(PluginSettings.IGNORE_REGISTRATION_POLICY) or False,
        }
    
    @access.admin
    @autoDescribeRoute(
        Description('Set OIDC configuration.')
        .param('keycloakUrl', 'Keycloak internal URL (for server-to-server communication)', dataType='string')
        .param('keycloakPublicUrl', 'Keycloak public URL (for browser redirects)', dataType='string')
        .param('keycloakRealm', 'Keycloak realm name', dataType='string')
        .param('clientId', 'OIDC client ID', dataType='string')
        .param('clientSecret', 'OIDC client secret', dataType='string')
        .param('enable', 'Enable OIDC authentication', dataType='boolean')
        .param('autoCreateUsers', 'Automatically create users', dataType='boolean')
        .param('allowRegistration', 'Allow registration from OIDC', dataType='boolean')
        .param('ignoreRegistrationPolicy', 'Ignore closed registration policy', dataType='boolean')
    )
    def setConfiguration(self, keycloakUrl, keycloakPublicUrl, keycloakRealm, clientId, clientSecret,
                        enable, autoCreateUsers, allowRegistration, ignoreRegistrationPolicy):
        """Set OIDC configuration (admin only)."""
        Setting().set(PluginSettings.KEYCLOAK_URL, keycloakUrl)
        Setting().set(PluginSettings.KEYCLOAK_PUBLIC_URL, keycloakPublicUrl)
        Setting().set(PluginSettings.KEYCLOAK_REALM, keycloakRealm)
        Setting().set(PluginSettings.CLIENT_ID, clientId)
        Setting().set(PluginSettings.CLIENT_SECRET, clientSecret)
        Setting().set(PluginSettings.ENABLE, enable)
        Setting().set(PluginSettings.AUTO_CREATE_USERS, autoCreateUsers)
        Setting().set(PluginSettings.ALLOW_REGISTRATION, allowRegistration)
        Setting().set(PluginSettings.IGNORE_REGISTRATION_POLICY, ignoreRegistrationPolicy)
        
        return self.getConfiguration()
    
    @access.public
    @autoDescribeRoute(
        Description('Get the Keycloak login URL.')
        .param('redirect', 'Where to redirect after login', dataType='string')
    )
    def getLoginUrl(self, redirect):
        """Get the Keycloak authorization URL."""
        try:
            if not Setting().get(PluginSettings.ENABLE):
                raise RestException('OIDC is not enabled', code=403)
            
            if not Setting().get(PluginSettings.CLIENT_ID):
                raise RestException('OIDC client ID is not configured', code=500)
            
            state = self._createStateToken(redirect)
            redirectUri = '/'.join((getApiUrl(), 'oidc', 'callback'))
            
            provider = KeycloakProvider()
            authUrl = provider.getAuthorizationUrl(state, redirectUri)
            
            return {'url': authUrl}
        except RestException:
            raise
        except Exception as e:
            raise RestException(f'Failed to get authorization URL: {str(e)}', code=502)
    
    
    @access.public
    @autoDescribeRoute(
        Description('OIDC callback endpoint.')
        .param('state', 'State token from authorization request', paramType='query')
        .param('code', 'Authorization code from Keycloak', paramType='query')
        .param('error', 'Error from Keycloak', paramType='query', required=False)
    )
    def callback(self, state, code, error=None):
        """Handle OIDC callback from Keycloak."""
        
        if error:
            raise RestException(f'OIDC error: {error}', code=400)
        
        if not state or not code:
            raise RestException('Missing state or code parameter', code=400)
        
        try:
            redirect = self._validateStateToken(state)
        except RestException as e:
            raise e
        
        try:
            provider = KeycloakProvider()
            redirectUri = '/'.join((getApiUrl(), 'oidc', 'callback'))
            
            # Exchange code for tokens
            tokenResp = provider.getToken(code, redirectUri)
            if 'access_token' not in tokenResp:
                raise RestException('No access token in token response', code=502)
            accessToken = tokenResp['access_token']
            
            # Get user info
            userInfo = provider.getUserInfo(accessToken)
            
            # Create or update Girder user
            user = provider.createOrUpdateUser(userInfo)
            
            # Create authentication token
            authToken = Token().createToken(user=user, days=365)
            
            # Redirect with token
            redirect = urlunparse(urlparse(redirect)._replace(
                query=urlencode({'girderToken': authToken['_id']})
            ))

            raise cherrypy.HTTPRedirect(redirect)
            
        except cherrypy.HTTPRedirect:
            raise
        except RestException as e:
            raise e
        except Exception as e:
            import traceback
            traceback.print_exc()
            raise RestException(f'OIDC authentication failed: {str(e)}', code=502)
    
    @access.public
    @autoDescribeRoute(
        Description('Check if the current user is an OIDC user.')
    )
    def isOidcUser(self):
        """Check if current user is authenticated via OIDC."""
        try:
            user = self.getCurrentUser()
            if not user:
                return {'isOidcUser': False}
            
            # Check if user has OIDC provider info
            oidcArray = user.get('oidc', [])
            isOidc = isinstance(oidcArray, list) and len(oidcArray) > 0 and any(
                o.get('provider') == 'keycloak' for o in oidcArray
            )
            
            return {'isOidcUser': isOidc}
        except Exception as e:
            raise RestException(f'Failed to check OIDC user status: {str(e)}', code=500)
    
    @access.public
    @autoDescribeRoute(
        Description('Exchange an OIDC access token for a Girder authentication token.')
        .param('access_token', 'OIDC access token (from direct access grant flow)', dataType='string')
    )
    def exchangeToken(self, access_token):
        """
        Exchange an OIDC access token (from direct access grant flow) for a Girder token.
        
        This endpoint allows clients to authenticate using OIDC direct access grant flow:
        1. Client authenticates with Keycloak using username/password
        2. Client receives OIDC access_token
        3. Client sends access_token to this endpoint
        4. Endpoint returns Girder authentication token
        
        Usage (with curl):
            # 1. Get OIDC token from Keycloak
            OIDC_TOKEN=$(curl -X POST https://keycloak/token \
              -d grant_type=password \
              -d client_id=girder \
              -d client_secret=secret \
              -d username=user \
              -d password=pass | jq -r .access_token)
            
            # 2. Exchange for Girder token
            curl -X POST https://girder/api/v1/oidc/token \
              -d access_token=$OIDC_TOKEN
        """
        if not access_token:
            raise RestException('Missing access_token parameter', code=400)
        
        if not Setting().get(PluginSettings.ENABLE):
            raise RestException('OIDC is not enabled', code=403)
        
        try:
            provider = KeycloakProvider()
            
            # Validate token and get user info
            userInfo = provider.getUserInfo(access_token)
            
            # Create or update Girder user
            user = provider.createOrUpdateUser(userInfo)
            
            # Create authentication token
            authToken = Token().createToken(user=user, days=0.25)
            
            return {
                'authToken': authToken['_id'],
                'userId': user['_id'],
                'login': user['login'],
                'email': user['email']
            }
            
        except RestException:
            raise
        except Exception as e:
            raise RestException(f'Token exchange failed: {str(e)}', code=502)



"""Tests for OIDC configuration endpoints."""

import pytest
from girder.models.setting import Setting
from girder.test import base
from girder.constants import AccessType

from girder_oidc.settings import PluginSettings


class TestOidcConfiguration(base.TestCase):
    """Test OIDC configuration endpoints."""

    def setUp(self):
        """Set up test fixtures."""
        super().setUp()
        self.admin_user = self.model('user').createUser(
            login='admin', email='admin@example.com', firstName='Admin',
            lastName='User', password='password')
        self.user = self.model('user').createUser(
            login='user', email='user@example.com', firstName='Regular',
            lastName='User', password='password')

    def testGetConfiguration(self):
        """Test GET /oidc/configuration returns current settings."""
        self.ensureUserIsAdmin(self.admin_user)
        
        # Set some configuration
        Setting().set(PluginSettings.KEYCLOAK_URL, 'https://keycloak:8443')
        Setting().set(PluginSettings.KEYCLOAK_PUBLIC_URL, 'https://localhost:8443')
        Setting().set(PluginSettings.KEYCLOAK_REALM, 'test-realm')
        Setting().set(PluginSettings.CLIENT_ID, 'test-client')
        Setting().set(PluginSettings.ENABLE, True)
        Setting().set(PluginSettings.AUTO_CREATE_USERS, True)
        
        resp = self.request('/oidc/configuration', user=self.admin_user)
        self.assertStatusOk(resp)
        
        config = resp.json
        self.assertEqual(config['keycloakUrl'], 'https://keycloak:8443')
        self.assertEqual(config['keycloakPublicUrl'], 'https://localhost:8443')
        self.assertEqual(config['keycloakRealm'], 'test-realm')
        self.assertEqual(config['clientId'], 'test-client')
        self.assertTrue(config['enable'])
        self.assertTrue(config['autoCreateUsers'])

    def testGetConfigurationForbidden(self):
        """Test regular users cannot get configuration."""
        resp = self.request('/oidc/configuration', user=self.user)
        self.assertStatus(resp, 403)

    def testGetConfigurationAnonymousForbidden(self):
        """Test anonymous users cannot get configuration."""
        resp = self.request('/oidc/configuration')
        self.assertStatus(resp, 401)

    def testSetConfiguration(self):
        """Test PUT /oidc/configuration updates settings."""
        self.ensureUserIsAdmin(self.admin_user)
        
        config_data = {
            'keycloakUrl': 'https://keycloak.example.com:8443',
            'keycloakPublicUrl': 'https://auth.example.com:8443',
            'keycloakRealm': 'production',
            'clientId': 'my-client',
            'clientSecret': 'secret123',
            'enable': True,
            'autoCreateUsers': False,
            'allowRegistration': True,
            'ignoreRegistrationPolicy': False
        }
        
        resp = self.request('/oidc/configuration', method='PUT',
                           user=self.admin_user, data=config_data)
        self.assertStatusOk(resp)
        
        # Verify settings were saved
        self.assertEqual(
            Setting().get(PluginSettings.KEYCLOAK_URL),
            'https://keycloak.example.com:8443'
        )
        self.assertEqual(
            Setting().get(PluginSettings.KEYCLOAK_PUBLIC_URL),
            'https://auth.example.com:8443'
        )
        self.assertEqual(
            Setting().get(PluginSettings.KEYCLOAK_REALM),
            'production'
        )
        self.assertFalse(Setting().get(PluginSettings.AUTO_CREATE_USERS))
        self.assertTrue(Setting().get(PluginSettings.ALLOW_REGISTRATION))

    def testSetConfigurationForbidden(self):
        """Test regular users cannot set configuration."""
        config_data = {
            'keycloakUrl': 'https://keycloak:8443',
            'keycloakPublicUrl': 'https://localhost:8443',
            'keycloakRealm': 'test',
            'clientId': 'test',
            'clientSecret': 'secret',
            'enable': True,
            'autoCreateUsers': True,
            'allowRegistration': False,
            'ignoreRegistrationPolicy': False
        }
        
        resp = self.request('/oidc/configuration', method='PUT',
                           user=self.user, data=config_data)
        self.assertStatus(resp, 403)

    def testConfigurationDefaults(self):
        """Test default configuration values."""
        self.ensureUserIsAdmin(self.admin_user)
        
        # Clear all settings
        for key in [PluginSettings.KEYCLOAK_URL, PluginSettings.KEYCLOAK_PUBLIC_URL,
                   PluginSettings.KEYCLOAK_REALM, PluginSettings.CLIENT_ID,
                   PluginSettings.ENABLE]:
            try:
                Setting().unset(key)
            except:
                pass
        
        resp = self.request('/oidc/configuration', user=self.admin_user)
        self.assertStatusOk(resp)
        
        config = resp.json
        # Should have defaults from environment or hardcoded values
        self.assertIsNotNone(config['keycloakUrl'])
        self.assertIsNotNone(config['keycloakPublicUrl'])
        self.assertIsNotNone(config['keycloakRealm'])



from setuptools import setup, find_packages

setup(
    name='girder-oidc',
    version='0.0.2',
    description='Girder plugin for OIDC/Keycloak authentication',
    author='Josselin Duchateau',
    license='Apache 2.0',
    packages=find_packages(),
    install_requires=[
        'girder>=3.0.0',
        'python-jose[cryptography]',
        'requests',
    ],
    entry_points={
        'girder.plugin': [
            'oidc = girder_oidc:OidcPlugin',
        ],
    },
    include_package_data=True,
)
